from unittest.mock import patch

import requests

from w.services.technical.models.request_response import RequestResponse
from w.tests.mixins.testcase_mixin import TestCaseMixin


class TestRequestResponse(TestCaseMixin):
    @classmethod
    def setup_class(cls):
        super().setup_class()
        cls.json_response = RequestResponse(
            requests.get("http://137.74.92.46:8888/get")
        )
        url = "https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf"
        cls.pdf_response = RequestResponse(requests.get(url))

    """
    get_header
    """

    def test_get_header_with_missing_header_return_none(self):
        """Ensure method return none if required header is missing"""
        assert self.json_response.get_header("unknown-header") is None

    def test_get_header_with_success_return_header(self):
        """Ensure method return content header"""
        assert self.json_response.get_header("content-type") == "application/json"

    """
    get_content_filename
    """

    def test_get_content_filename_with_failure_return_none(self):
        """Ensure method return None if we cannot guess filename"""
        assert self.json_response.get_content_filename() is None

    def test_get_content_filename_from_content_disposition_header_return_filename(self):
        """Ensure method return filename"""
        target = (
            "w.tests.services.technical.models.test_request_response"
            ".RequestResponse.get_header"
        )
        content_dispositions = [
            "dummy content",
            'attachment; filename="a-file.pdf"',
        ]
        with patch(target, side_effect=content_dispositions):
            assert self.pdf_response.get_content_filename() is None
            assert self.pdf_response.get_content_filename() == "a-file.pdf"

    def test_get_content_filename_from_url_return_filename(self):
        """Ensure method return None if we cannot guess filename"""
        assert self.pdf_response.get_content_filename() == "dummy.pdf"

    """
    as_attachment
    """

    def test_as_attachment_with_no_content_disposition_header_return_false(self):
        """Ensure method return false is missing content-disposition header"""
        assert self.json_response.as_attachment() is False

    def test_as_attachment_with_attachment_content_disposition_return_true(self):
        """Ensure method return True if attachment"""
        target = (
            "w.tests.services.technical.models.test_request_response"
            ".RequestResponse.get_header"
        )
        content_dispositions = [
            'attachment; filename="a-file.pdf"',
        ]
        with patch(target, side_effect=content_dispositions):
            assert self.pdf_response.as_attachment() is True

    def test_as_attachment_with_inline_content_disposition_return_true(self):
        """Ensure method return None if we cannot guess filename"""
        target = (
            "w.tests.services.technical.models.test_request_response"
            ".RequestResponse.get_header"
        )
        content_dispositions = [
            'inline; filename="a-file.pdf"',
        ]
        with patch(target, side_effect=content_dispositions):
            assert self.pdf_response.as_attachment() is False
